--[[
  Mitchell's editing.lua
  Copyright (c) 2006-2007 Mitchell Foral. All rights reserved.

  SciTE-tools homepage: http://caladbolg.net/scite.php
  Send email to: mitchell<att>caladbolg<dott>net

  Permission to use, copy, modify, and distribute this file
  is granted, provided credit is given to Mitchell.

  Editing commands for the SciTE "bundle"
]]--

-- for kill-ring
local kill_ring = { pos = 1, maxn = 10 }

-- for matching enclosures
char_matches = {
  ['('] = ')', ['['] = ']', ['{'] = '}',
  ["'"] = "'" , ['"'] = '"'
}

-- for enclosing text or selecting text in enclosures
-- note chars and tag enclosures are generated on the fly
enclosure = {
  dbl_quotes = { left = '"', right = '"' },
  sng_quotes = { left = "'", right = "'" },
  parens     = { left = '(', right = ')' },
  brackets   = { left = '[', right = ']' },
  braces     = { left = '{', right = '}' },
  chars      = { left = ' ', right = ' ' },
  tags       = { left = '<', right = '>' },
  tag        = { left = ' ', right = ' ' },
  single_tag = { left = '<', right = ' />' }
}

revenclosure = {
  ['('] = 'parens',
  [')'] = 'parens',
  ['['] = 'brackets',
  [']'] = 'brackets',
  ['{'] = 'braces',
  ['}'] = 'braces'
}

-- local functions
local insert_into_kill_ring, scroll_kill_ring
local get_preceding_number, get_sel_or_line

Editing = {}

function Editing.SetupTabs()
  local lexerName = Global.GetLexerName()
  if lexerName then
    if props["tabsize." .. lexerName]~="" then
      editor.TabWidth = props["tabsize." .. lexerName]
    else
      editor.TabWidth = props["tabsize"]
    end
    if props["indent.size." .. lexerName]~="" then
      editor.Indent   = props["indent.size." .. lexerName]
    else
      editor.Indent   = props["indent.size"]
    end
    if props["use.tabs." .. lexerName]~="" then
      editor.UseTabs  = props["use.tabs." .. lexerName] == 1
    else
      editor.UseTabs  = props["use.tabs"]
    end
    if props["find.files." .. lexerName]~="" then
      props["find.files"] = props["find.files." .. lexerName]
    end
  end
end

-- selects to end of line if no text is currently selected and
-- pushes it to the kill-ring before cutting (default) or copying
function Editing.Smart_CutCopy(action)
  local txt = editor:GetSelText()
  if string.len(txt) == 0 then return end
  txt = editor:GetSelText()
  insert_into_kill_ring(txt)
  kill_ring.pos = 1
  if action ~= 'copy' then editor:Cut() return end
  editor:Copy()
end

-- retrieves top item off kill-ring, or scrolls through it
-- if no args, does standard paste from current item in kill-ring
-- if arg is 'cycle', starts cycling through previous cuts/copies
-- if arg is 'reverse', cycles in reverse order
function Editing.Smart_Paste(...)
  local anchor, pos = editor.Anchor, editor.CurrentPos
  if pos < anchor then anchor = pos end
  local txt = editor:GetSelText()
  if txt == kill_ring[kill_ring.pos] then
    scroll_kill_ring( arg[1] )
  end

  -- if text was copied to the clipboard from other apps, insert
  -- it into the kill-ring so it can be pasted (thanks to Nathan
  -- Robinson)

--~   local clip_txt, found = scite.GetClipboardText(), false
--~   if clip_txt ~= '' then
--~     for _, ring_txt in ipairs(kill_ring) do
--~       if clip_txt == ring_txt then found = true break end
--~     end
--~     if not found then insert_into_kill_ring(clip_txt) end
--~   end

  txt = kill_ring[kill_ring.pos]
  if txt then
    editor:ReplaceSel(txt)
    if arg[1] then editor.Anchor = anchor end -- cycle
  end
end

-- encloses text in an enclosure set
-- if there is text selected, it is enclosed; otherwise the
-- previous word is enclosed
-- multiple previous words can be enclosed by appending a number
-- to the end of the last word (e.g. enclose this2); if enclosing
-- with multiple characters, the same rule applies (e.g. this**2)
function Editing.Enclose(str)
  editor:BeginUndoAction()
  local txt = editor:GetSelText()
  if txt == '' then
    if str == 'chars' then
      local num_chars, len_num_chars = get_preceding_number()
      for i = 1, len_num_chars do editor:DeleteBack() end
      for i = 1, num_chars do editor:CharLeftExtend() end
      enclosure[str].left  = editor:GetSelText()
      enclosure[str].right = enclosure[str].left
      editor:DeleteBack()
    end
    local num_words, len_num_chars = get_preceding_number()
    for i = 1, len_num_chars do editor:DeleteBack() end
    for i = 1, num_words do editor:WordLeftExtend() end
    txt = editor:GetSelText()
  end
  local len = 0
  if str == 'tag' then
    enclosure[str].left  = '<'..txt..'>'
    enclosure[str].right = '</'..txt..'>'
    len = string.len(txt) + 3 txt = ''
  end
  local left  = enclosure[str].left
  local right = enclosure[str].right
  editor:ReplaceSel(left..txt..right)
  if str == 'tag' then editor:GotoPos(editor.CurrentPos - len) end
  editor:EndUndoAction()
end

-- inserts text into the kill_ring
insert_into_kill_ring = function(txt)
  table.insert(kill_ring, 1, txt)
  if table.getn(kill_ring) > kill_ring.maxn then
    table.remove(kill_ring, kill_ring.maxn + 1)
  end
end

-- scrolls kill_ring backward or forward (default)
scroll_kill_ring = function(direction)
  if direction == 'reverse' then
    kill_ring.pos = kill_ring.pos - 1
    if kill_ring.pos < 1 then
      kill_ring.pos = table.getn(kill_ring)
    end
  else
    kill_ring.pos = kill_ring.pos + 1
    if kill_ring.pos > table.getn(kill_ring) then
      kill_ring.pos = 1
    end
  end
end

-- returns number to the left of the cursor
get_preceding_number = function()
  local pos = editor.CurrentPos
  local char = editor.CharAt[pos - 1]
  local txt = ''
  while tonumber( string.char(char) ) do
    txt = txt..string.char(char)
    pos = pos - 1
    char = editor.CharAt[pos - 1]
  end
  return tonumber(txt) or 1, string.len(txt)
end

-- returns selection or current line
get_sel_or_line = function()
  if editor:GetSelText() == '' then Editing.select_line() end
  return editor:GetSelText()
end
